function [x,objval,sqslack,LPslack] = solve_eq(N_alphas, N_prefcases, H_blk, H_wht, startH_full, H_list_full)
% Solves QP to determine whether payoff parameters yield equilibrium.
% - construct constraints for LP and QP problems
% - solve preliminary LP problem
% - construct inequality constraints for QP problem
% - solve QP problem, using multiple starting values if needed
% - return large value if problem fails to solve



%% PROGRAM SETUP

% Define global variables:

global N N_B N_W N_types_ego draws obs_shares LPopts



%% CONSTRUCT CONSTRAINTS

% A) Box Bounds on Alphas and Slacks

ub = vertcat(  ones(N_alphas,1), ones(N_types_ego,1)/N*2, ones(N_types_ego,1)/N/2 );
lb = zeros(N_alphas + 2*N_types_ego,1);

% B) Alphas Add to 1

% Storage objects:

a = ones(N_prefcases,1);
A = sparse([],[],true,N_prefcases,N_alphas,N_alphas);

% Populate rows for each preference class:

for H = 2:(length(startH_full)-1)
    first = startH_full(H);
    last = startH_full(H+1)-1;
    %
    A(H,first:last) = true;
end

% First and last preference classes:

A(1,1) = true;

H = N_prefcases;
    first = startH_full(H);
    last = N_alphas;
    A(H,first:last) = true;

% C) Match Type Shares

% Compute pref. class probabilities:

for H = 1:length(H_blk)
    H_blk(H).prob = ones(1,length(H_blk(H).list)) ...
        * H_blk(H).freq/draws * N_B/N;
end

for H = 1:length(H_wht)
    H_wht(H).prob = ones(1,length(H_wht(H).list)) ...
        * H_wht(H).freq/draws * N_W/N;
end

% Define vector of pref. class probabilities, repeated to match size of each class:

clear global muXpH_vec
global muXpH_vec

muXpH_vec = [ [H_blk.prob], [H_wht.prob] ];

% Define matrix to compute type measures from vector of allocation parameters:

muXpH_mat = sparse(double(H_list_full),1:N_alphas,muXpH_vec, N_types_ego,N_alphas);

% D) Combine Equality Constraints

a_eq = vertcat(a, obs_shares);
A_eq = vertcat( [A sparse(N_prefcases,2*N_types_ego)], ...
                [muXpH_mat, speye(N_types_ego), -speye(N_types_ego)]  );



%% PRELIMINARY LP TO MATCH TYPE SHARES

% Objective - add up slack variables:

addslacks = sparse(N_alphas + (1:2*N_types_ego),1,1);

% Solve LP problem:

[startpt,LPslack,LPexitflag,LPoutput] = linprog(addslacks,[],[],A_eq,a_eq,lb,[],[],LPopts);

% Define constraints on sums of slack variables, for QP problem:

totslack = max(LPslack/2 + 1/2/N, 6/N)

a_in = vertcat( 2*totslack, totslack );
A_in = vertcat( [sparse(1, N_alphas) ones(1,N_types_ego) sparse(1,N_types_ego)], ...
                [sparse(1, N_alphas) sparse(1,N_types_ego) ones(1,N_types_ego)] );



%% SOLVE QP PROBLEM

% Regular starting point:

[x,objval,QPexitflag,QPoutput,lambda] = knitromatlab(@obj_fn,startpt,A_in,a_in,A_eq,a_eq,lb,ub,[],[],[],'test.opt');

% Try other starting points if poor result:

if QPexitflag <= -200
    rndstart = 0.9 + 0.2 * rand(size(x)) .* x;
    [x,objval,QPexitflag,QPoutput,lambda] = knitromatlab(@obj_fn,rndstart,A_in,a_in,A_eq,a_eq,lb,ub,[],[],[],'mult1.opt');
end

if objval > 1e-4
    rndstart = 0.9 + 0.2 * rand(size(x)) .* x;
    [x,objval,QPexitflag,QPoutput,lambda] = knitromatlab(@obj_fn,rndstart,A_in,a_in,A_eq,a_eq,lb,ub,[],[],[],'mult2.opt');
end

% Compute sum of squared slacks:

slacks = x(N_alphas + (1:2*N_types_ego));

sqslack = sum(slacks.^2);

% Values if QP fails to match observed shares:

if QPexitflag == -204 || QPexitflag == -205
    x = [];
    objval = 1/2;
    sqslack = 1/2;
end



end
