# Setup for microsimulation procedure:
# - network size, distribution of characteristics
# - set preference parameters and draw preference shocks
# - simulation procedure parameters
# - draw random initial graphs



### PROGRAM SETUP

library(Matrix)



### POPULATION AND CHARACTERISTICS

# Network size (numbers of blacks and whites):

N.B <- 500
N.W <- 600
N <- N.B + N.W

# Cardinality of predetermined characteristics ( |X| ):

chars <- 2

# Indices for predetermined characteristics:

blacks <- 1:N.B
whites <- (N.B+1):N

# Indicators for predetermined characteristics:

black <- white <- logical(N)
black[blacks] <- T
white[whites] <- T

# Number of networks to simulate:

sims <- 50



### PREFERENCES

# Payoff depth:

D <- 1

# Number of links:

L <- 1

# Preference parameters:

param <- c(0.40,0.20,0.15,0.50, 0, 0)

f.BB <- param[1]
f.BW <- param[2]
f.WB <- param[3]
f.WW <- param[4]
v <- param[5]
w <- param[6]

# Vectors of relevant payoff parameters for having a same or different race alter:
# (for programming convenience later)

f.same <- f.BB * black  +  f.WW * white

f.diff <- (f.BW - f.BB) * black  +  (f.WB - f.WW) * white

# Preference shocks:

set.seed(7384)

Draws.sim <- array(runif(sims*N*L*chars), dim=c(sims,N,L*chars))

# Sort shocks within each alter characteristic (discussed in D.3):

for (s in 1:sims)  {  for (i in 1:N)  {
  Draws.sim[s,i,1:L] <- sort(Draws.sim[s,i,1:L], decreasing=TRUE)
  Draws.sim[s,i,L+1:L] <- sort(Draws.sim[s,i,L+1:L], decreasing=TRUE)
  }  }



### SIMULATION TOOLS

# A) Simulation Parameters

# Loop cutoffs:

max.nets <- 7  #  number of random initial graphs
num.seqs <- 100  #  number of loops through the network 

# B) Sampling Function Wrapper

# Resample function:
# (Source: modified from R help file for 'sample')

resample <- function(x, size, ...)  {
  if(length(x)==1) x else sample(x, size, ...)  }

# C) Draw Initial Graphs

# Link probability:

p.link <- 1/N

# Storage object:

G.init.list <- list()

# Start of loop to draw the initial graphs:

for (s in 1:sims)  {

  G.init.list <- c(G.init.list, list(as.list(1:max.nets)) )

# Each simulated network has 'max.nets' random initial graphs:

for (g in 1:max.nets)  {

# First draw the total number of links in the graph:

N.links <- rbinom(1, size = N*(N-1)/2, prob = p.link)

# Then draw a list of node pairs for this number of links:

G.init.list[[s]][[g]] <- 
  cbind( sample(x=N, size=N.links, replace=TRUE), sample(x=N, size=N.links, replace=TRUE) )

# Define adjacency matrix (as logical):

G <- matrix(FALSE, N, N)

G[ G.init.list[[s]][[g]] ] <- TRUE

G[t(G)] <- TRUE

# Remove links to satisfy bounded degree:

while (max(rowSums(G) > L) )  {
  for (i in resample(which(rowSums(G) > L) ) )  {
    if (sum(G[i,]) > L)  {
      j <- resample(which(as.logical(G[i,])),1)
      G[i,j] <- G[j,i] <- FALSE
  } } } 

# Remove diagonal elements:

diag(G) <- FALSE

# Replace list of links in storage object:

G <- as(G, "sparseMatrix")

G.init.list[[s]][[g]] <- as.matrix( summary(G)[,c("i","j")] )

# End loop:

} }

# Clean up:

rm(G)
