# Enumerate types (i.e., construct list of (A,v) representations) for D=1, L=1
# - construct non-isomorphic adjacency matrices
# - generate vectors of possible node characteristics
# - combine the above and check for isomorphisms to compile the list
# - draw preference shocks used later to simulate preference class probabilities
# - all lines related to multiple alters and indirect alters are removed



### PROGRAM SETUP

library(abind)



### ADJACENCY MATRICES

# Size of adjacency matrix:

A.size <- 1
  for ( d in 1:D ) A.size <- A.size + L*(L-1)^(d-1)

# Start list of non-isomorphic adj. matrices with the isolated type:

A.list <- matrix(F,nrow=A.size,ncol=A.size)

# A) Loop to Construct Matrices

# Loop over number of links of the ego (l.e):
  for (l.e in 1:L)  {

# A.1) No Mutual Friends

# Loop over number of additional alters of the first potential direct alter (l.1):
  for (l.1 in 0:(if (l.e >= 1) L-1 else 0) )  {

# First define empty adjacency matrix:

A.t <- matrix(F,nrow=A.size,ncol=A.size)

# Add ego's links:

A.t[1, 1 + 1:l.e] <- T

# Add direct alters' other links:

if (l.1 > 0) A.t[1+1, 1 + L + 0*(L-1) + 1:l.1] <- T

# Append new A to list of adjacency matrices:

A.list <- abind(A.list,A.t,along=3)
 
}  #  END: loop over direct alter

# A.2) One Mutual Friendship
# (link between first and second direct alters)

# N/A

# A.3) Two Mutual Friendships
# (first and second direct alters are both linked to the third)

# N/A

# A.3) Three Mutual Friendships
# (all three direct alters are mutually linked)

# N/A

} # END: loop over l.e

# Number of distinct adjacency matrices:

N.A <- dim(A.list)[3]

###

# B) Make Matrices Symmetric

for (A in 1:N.A)  {
  A.t <- A.list[,,A]
  A.list[,,A] <- A.t | t(A.t)
  }



### NODE CHARACTERISTICS

# A) Characteristics of Direct Alters

# Define storage object:

v.list <- numeric(0)

# Loop over characteristics:

# Ego is left blank:
  v1 <- 0
# First direct alter:
  for (v2 in 1:chars)  {

# Append to list of vectors of characteristics:

v.list <- rbind(v.list, c(v1,v2) )

}  #  END: loops over characteristics of direct alters

# Number of different vectors of characteristics of direct alters:

N.v <- nrow(v.list)

# B) Add Characteristic(s) of Ego

# Define storage object:

v.list.ego <- numeric(0)

# Loop over characteristics of ego:

for (x in 1:chars)  {
  v.list.add <- v.list
  v.list.add[,1] <- x
  v.list.ego <- rbind(v.list.ego, v.list.add)
}



### ENUMERATE TYPES W/OUT EGO CHARACTERISTICS

# A) Permutations of Direct Alters
# (to assess isomorphisms)

# N/A

# B) Loop over A's and v's to Make Enumeration

# Note: This makes a list of the indices for A and v from A.list and v.list.

# Start with isolated type to define storage object:

t.list <- rbind(c(1,1) )

colnames(t.list) <- c("A","v")

# Start loop:

for (A in 2:N.A)  {

for (v in 1:N.v)  {

# Use elementwise product to indicate node characteristics:

Av <- A.list[,,A] * v.list[v,]

# C) Check for Isomorphisms

# Indicator for isomorphism found:

same <- F

# Loop over previous vectors of alter characteristics:
  if (v > 1)  {  for (w in 1:(v-1) )  {  

# Check for match:

if (!same)  {
  Aw <- A.list[,,A] * v.list[w,]
  if (all(Aw==Av) )  same <- T

}  # END: if statements for 'same' already found

} }  #  END: loop over previous vectors of alter characteristics

  if (!same) t.list <- rbind(t.list, c(A,v) )

} }

# Type indices (without ego characteristics):

N.types <- nrow(t.list)
t.index <- 1:N.types



### ENUMERATE TYPES WITH EGO CHARACTERISTICS

# Previous list provides list for first value of ego characteristic:
# (using v.list.ego rather than v.list)

t.list.ego <- t.list

# Loop over remaining possible characteristics of ego:

for (x in 1:(chars-1))  {
  t.list.add <- t.list
  t.list.add[,"v"] <- t.list[,"v"] + N.v*x
  t.list.ego <- rbind(t.list.ego, t.list.add)
}

# Type indices (with ego characteristics):

N.types.ego <- nrow(t.list.ego)
t.index.ego <- 1:N.types.ego



### PREFERENCE SHOCKS

# Number of draws and random seed:

draws <- 10000

set.seed(746257)

# Draw shocks:

Shocks <- array(runif(draws*L*chars), dim=c(draws,chars,L))
  dimnames(Shocks)[[2]] <- c("B","W")

# Sort shocks within each alter characteristic:
# Note: See Appendix D.3 for discussion.

for (s in 1:draws)  {
  Shocks[s,"B",1:L] <- sort(Shocks[s,"B",1:L], decreasing=TRUE)
  Shocks[s,"W",1:L] <- sort(Shocks[s,"W",1:L], decreasing=TRUE)
  }

# Take sum of shocks within each characteristic:
# (only the sum is needed to compute utility)

Shocksum <- Shocks

for (l in 1:L)  {
  Shocksum[,"B",l] <- rowSums(cbind(Shocks[,"B",1:l]) ) 
  Shocksum[,"W",l] <- rowSums(cbind(Shocks[,"W",1:l]) ) 
  }

# Add a column for zero shocks (no direct alters with those characteristics):

Shocksum <- abind(matrix(0,draws,chars),Shocksum,along=3)

# Clean up:

rm(Shocks)
